@@ -25,7 +25,7 @@ from docutils.frontend import OptionParser
 from docutils.readers.doctree import Reader as DoctreeReader
 
 from sphinx import addnodes
-from sphinx.util import (get_matching_docs, ensuredir, relative_uri, SEP, os_path)
+from sphinx.util import mtimes_of_files, ensuredir, relative_uri, SEP, os_path
 from sphinx.htmlhelp import build_hhx
 from sphinx.htmlwriter import HTMLWriter, HTMLTranslator, SmartyPantsHTMLTranslator
 from sphinx.latexwriter import LaTeXWriter
@@ -75,7 +75,7 @@ class Builder(object):
 
     def init_templates(self):
         """Call if you need Jinja templates in the builder."""
-        # lazily import this, maybe other builders won't need it
+        # lazily import this, other builders won't need it
         from sphinx._jinja import Environment, SphinxFileSystemLoader
 
         # load templates
@@ -83,6 +83,7 @@ class Builder(object):
         base_templates_path = path.join(path.dirname(__file__), 'templates')
         ext_templates_path = [path.join(self.srcdir, dir)
                               for dir in self.config.templates_path]
+        self.templates_path = [base_templates_path] + ext_templates_path
         loader = SphinxFileSystemLoader(base_templates_path, ext_templates_path)
         self.jinja_env = Environment(loader=loader,
                                      # disable traceback, more likely that something
@@ -103,14 +104,18 @@ class Builder(object):
         raise NotImplementedError
 
     def get_relative_uri(self, from_, to, typ=None):
-        """Return a relative URI between two source filenames.
-           May raise environment.NoUri if there's no way to return a
-           sensible URI."""
+        """
+        Return a relative URI between two source filenames. May raise environment.NoUri
+        if there's no way to return a sensible URI.
+        """
         return relative_uri(self.get_target_uri(from_),
                             self.get_target_uri(to, typ))
 
     def get_outdated_docs(self):
-        """Return a list of output files that are outdated."""
+        """
+        Return an iterable of output files that are outdated, or a string describing
+        what an update build will build.
+        """
         raise NotImplementedError
 
     def status_iterator(self, iterable, summary, colorfunc):
@@ -127,8 +132,7 @@ class Builder(object):
     # build methods
 
     def load_env(self):
-        """Set up the build environment. Return True if a pickled file could be
-           successfully loaded, False if a new environment had to be created."""
+        """Set up the build environment."""
         if self.env:
             return
         if not self.freshenv:
@@ -143,8 +147,10 @@ class Builder(object):
                 else:
                     self.info('failed: %s' % err)
                 self.env = BuildEnvironment(self.srcdir, self.doctreedir, self.config)
+                self.env.find_files(self.config)
         else:
             self.env = BuildEnvironment(self.srcdir, self.doctreedir, self.config)
+            self.env.find_files(self.config)
         self.env.set_warnfunc(self.warn)
 
     def build_all(self):
@@ -171,12 +177,8 @@ class Builder(object):
     def build_update(self):
         """Only rebuild files changed or added since last build."""
         to_build = self.get_outdated_docs()
-        if not to_build and self.env.all_docs:
-            # if there is nothing in all_docs, it's a fresh env
-            self.info(bold('no target files are out of date, exiting.'))
-            return
         if isinstance(to_build, str):
-            self.build([], to_build)
+            self.build(['__all__'], to_build)
         else:
             to_build = list(to_build)
             self.build(to_build,
@@ -213,6 +215,10 @@ class Builder(object):
             # global actions
             self.info(bold('checking consistency...'))
             self.env.check_consistency()
+        else:
+            if method == 'update' and not docnames:
+                self.info(bold('no targets are out of date.'))
+                return
 
         # another indirection to support methods which don't build files
         # individually
@@ -222,14 +228,16 @@ class Builder(object):
         self.info(bold('finishing... '))
         self.finish()
         if self.app._warncount:
-            self.info(bold('build succeeded, %s warnings.' % self.app._warncount))
+            self.info(bold('build succeeded, %s warning%s.' %
+                           (self.app._warncount,
+                            self.app._warncount != 1 and 's' or '')))
         else:
             self.info(bold('build succeeded.'))
 
     def write(self, build_docnames, updated_docnames, method='update'):
         if build_docnames is None:
             # build_all
-            build_docnames = self.env.all_docs
+            build_docnames = self.env.found_docs
         if method == 'update':
             # build updated ones as well
             docnames = set(build_docnames) | set(updated_docnames)
@@ -271,8 +279,9 @@ class StandaloneHTMLBuilder(Builder):
     Builds standalone HTML docs.
     """
     name = 'html'
-
     copysource = True
+    out_suffix = '.html'
+    indexer_format = 'json'
 
     def init(self):
         """Load templates."""
@@ -339,28 +348,40 @@ class StandaloneHTMLBuilder(Builder):
         destination = StringOutput(encoding='utf-8')
         doctree.settings = self.docsettings
 
+        self.imgpath = relative_uri(self.get_target_uri(docname), '_images')
         self.docwriter.write(doctree, destination)
         self.docwriter.assemble_parts()
 
         prev = next = None
         parents = []
         related = self.env.toctree_relations.get(docname)
+        titles = self.env.titles
         if related:
-            prev = {'link': self.get_relative_uri(docname, related[1]),
-                    'title': self.render_partial(self.env.titles[related[1]])['title']}
-            next = {'link': self.get_relative_uri(docname, related[2]),
-                    'title': self.render_partial(self.env.titles[related[2]])['title']}
+            try:
+                prev = {'link': self.get_relative_uri(docname, related[1]),
+                        'title': self.render_partial(titles[related[1]])['title']}
+            except KeyError:
+                # the relation is (somehow) not in the TOC tree, handle that gracefully
+                prev = None
+            try:
+                next = {'link': self.get_relative_uri(docname, related[2]),
+                        'title': self.render_partial(titles[related[2]])['title']}
+            except KeyError:
+                next = None
         while related:
-            parents.append(
-                {'link': self.get_relative_uri(docname, related[0]),
-                 'title': self.render_partial(self.env.titles[related[0]])['title']})
+            try:
+                parents.append(
+                    {'link': self.get_relative_uri(docname, related[0]),
+                     'title': self.render_partial(titles[related[0]])['title']})
+            except KeyError:
+                pass
             related = self.env.toctree_relations.get(related[0])
         if parents:
             parents.pop() # remove link to the master file; we have a generic
                           # "back to index" link already
         parents.reverse()
 
-        title = self.env.titles.get(docname)
+        title = titles.get(docname)
         if title:
             title = self.render_partial(title)['title']
         else:
@@ -383,7 +404,7 @@ class StandaloneHTMLBuilder(Builder):
         self.handle_page(docname, ctx)
 
     def finish(self):
-        self.info(bold('writing additional files...'))
+        self.info(bold('writing additional files...'), nonl=1)
 
         # the global general index
 
@@ -397,6 +418,7 @@ class StandaloneHTMLBuilder(Builder):
             genindexentries = self.env.index,
             genindexcounts = indexcounts,
         )
+        self.info(' genindex', nonl=1)
         self.handle_page('genindex', genindexcontext, 'genindex.html')
 
         # the global module index
@@ -442,23 +464,39 @@ class StandaloneHTMLBuilder(Builder):
                 modindexentries = modindexentries,
                 platforms = platforms,
             )
+            self.info(' modindex', nonl=1)
             self.handle_page('modindex', modindexcontext, 'modindex.html')
 
         # the search page
+        self.info(' search', nonl=1)
         self.handle_page('search', {}, 'search.html')
 
         # additional pages from conf.py
         for pagename, template in self.config.html_additional_pages.items():
+            self.info(' '+pagename, nonl=1)
             self.handle_page(pagename, {}, template)
 
         # the index page
         indextemplate = self.config.html_index
         if indextemplate:
+            self.info(' index', nonl=1)
             self.handle_page('index', {'indextemplate': indextemplate}, 'index.html')
 
+        self.info()
+
+        # copy image files
+        if self.env.images:
+            self.info(bold('copying images...'), nonl=1)
+            ensuredir(path.join(self.outdir, '_images'))
+            for src, (_, dest) in self.env.images.iteritems():
+                self.info(' '+src, nonl=1)
+                shutil.copyfile(path.join(self.srcdir, src),
+                                path.join(self.outdir, '_images', dest))
+            self.info()
+
         # copy static files
         self.info(bold('copying static files...'))
-        ensuredir(path.join(self.outdir, 'static'))
+        ensuredir(path.join(self.outdir, '_static'))
         staticdirnames = [path.join(path.dirname(__file__), 'static')] + \
                          [path.join(self.srcdir, spath)
                           for spath in self.config.html_static_path]
@@ -466,41 +504,38 @@ class StandaloneHTMLBuilder(Builder):
             for filename in os.listdir(staticdirname):
                 if not filename.startswith('.'):
                     shutil.copyfile(path.join(staticdirname, filename),
-                                    path.join(self.outdir, 'static', filename))
+                                    path.join(self.outdir, '_static', filename))
         # add pygments style file
-        f = open(path.join(self.outdir, 'static', 'pygments.css'), 'w')
+        f = open(path.join(self.outdir, '_static', 'pygments.css'), 'w')
         f.write(PygmentsBridge('html', self.config.pygments_style).get_stylesheet())
         f.close()
 
         # dump the search index
         self.handle_finish()
 
-    # --------- these are overwritten by the Pickle builder
-
-    def get_target_uri(self, docname, typ=None):
-        return docname + '.html'
-
     def get_outdated_docs(self):
-        for docname in get_matching_docs(
-            self.srcdir, self.config.source_suffix,
-            exclude=set(self.config.unused_docs),
-            prune=['_sources']):
-            targetname = self.env.doc2path(docname, self.outdir, '.html')
-            try:
-                targetmtime = path.getmtime(targetname)
-            except:
-                targetmtime = 0
+        if self.templates_path:
+            template_mtime = max(mtimes_of_files(self.templates_path, '.html'))
+        else:
+            template_mtime = 0
+        for docname in self.env.found_docs:
             if docname not in self.env.all_docs:
                 yield docname
-            elif path.getmtime(self.env.doc2path(docname)) > targetmtime:
+                continue
+            targetname = self.env.doc2path(docname, self.outdir, self.out_suffix)
+            try:
+                targetmtime = path.getmtime(targetname)
+            except Exception:
+                targetmtime = 0
+            srcmtime = max(path.getmtime(self.env.doc2path(docname)), template_mtime)
+            if srcmtime > targetmtime:
                 yield docname
 
-
     def load_indexer(self, docnames):
         try:
-            f = open(path.join(self.outdir, 'searchindex.json'), 'r')
+            f = open(path.join(self.outdir, 'searchindex.'+self.indexer_format), 'r')
             try:
-                self.indexer.load(f, 'json')
+                self.indexer.load(f, self.indexer_format)
             finally:
                 f.close()
         except (IOError, OSError):
@@ -513,6 +548,11 @@ class StandaloneHTMLBuilder(Builder):
         if self.indexer is not None and title:
             self.indexer.feed(pagename, title, doctree)
 
+    # --------- these are overwritten by the Pickle builder
+
+    def get_target_uri(self, docname, typ=None):
+        return docname + '.html'
+
     def handle_page(self, pagename, addctx, templatename='page.html'):
         ctx = self.globalcontext.copy()
         ctx['current_page_name'] = pagename
@@ -561,22 +601,13 @@ class PickleHTMLBuilder(StandaloneHTMLBuilder):
     Builds HTML docs without rendering templates.
     """
     name = 'pickle'
+    out_suffix = '.fpickle'
+    indexer_format = 'pickle'
 
     def init(self):
         self.init_translator_class()
-
-    def get_outdated_docs(self):
-        for docname in get_matching_docs(
-            self.srcdir, self.config.source_suffix,
-            exclude=set(self.config.unused_docs),
-            prune=['_sources']):
-            targetname = self.env.doc2path(docname, self.outdir, '.fpickle')
-            try:
-                targetmtime = path.getmtime(targetname)
-            except:
-                targetmtime = 0
-            if path.getmtime(self.env.doc2path(docname)) > targetmtime:
-                yield docname
+        # no templates used, but get_outdated_docs() needs this attribute
+        self.templates_path = []
 
     def get_target_uri(self, docname, typ=None):
         if docname == 'index':
@@ -585,23 +616,6 @@ class PickleHTMLBuilder(StandaloneHTMLBuilder):
             return docname[:-5] # up to sep
         return docname + SEP
 
-    def load_indexer(self, docnames):
-        try:
-            f = open(path.join(self.outdir, 'searchindex.pickle'), 'r')
-            try:
-                self.indexer.load(f, 'pickle')
-            finally:
-                f.close()
-        except (IOError, OSError):
-            pass
-        # delete all entries for files that will be rebuilt
-        self.indexer.prune(set(self.env.all_docs) - set(docnames))
-
-    def index_page(self, pagename, doctree, title):
-        # only index pages with title
-        if self.indexer is not None and title:
-            self.indexer.feed(pagename, title, doctree)
-
     def handle_page(self, pagename, ctx, templatename='page.html'):
         ctx['current_page_name'] = pagename
         sidebarfile = self.config.html_sidebars.get(pagename, '')
@@ -618,7 +632,7 @@ class PickleHTMLBuilder(StandaloneHTMLBuilder):
         # if there is a source file, copy the source file for the
         # "show source" link
         if ctx.get('sourcename'):
-            source_name = path.join(self.outdir, 'sources',
+            source_name = path.join(self.outdir, '_sources',
                                     os_path(ctx['sourcename']))
             ensuredir(path.dirname(source_name))
             shutil.copyfile(self.env.doc2path(pagename), source_name)
@@ -783,6 +797,15 @@ class LaTeXBuilder(Builder):
         return largetree
 
     def finish(self):
+        # copy image files
+        if self.env.images:
+            self.info(bold('copying images...'), nonl=1)
+            for src, (_, dest) in self.env.images.iteritems():
+                self.info(' '+src, nonl=1)
+                shutil.copyfile(path.join(self.srcdir, src),
+                                path.join(self.outdir, dest))
+            self.info()
+
         self.info(bold('copying TeX support files...'))
         staticdirname = path.join(path.dirname(__file__), 'texinputs')
         for filename in os.listdir(staticdirname):
