@@ -191,6 +191,7 @@ c_funcptr_sig_re = re.compile(
         (\( [^()]+ \)) \s* # name in parentheses
         \( (.*) \) $       # arguments
     ''', re.VERBOSE)
+c_funcptr_name_re = re.compile(r'^\(\s*\*\s*(.*?)\s*\)$')
 
 # RE to split at word boundaries
 wsplit_re = re.compile(r'(\W+)')
@@ -224,6 +225,10 @@ def parse_c_signature(signode, sig, desctype):
     signode += addnodes.desc_type("", "")
     parse_c_type(signode[-1], rettype)
     signode += addnodes.desc_name(name, name)
+    # clean up parentheses from canonical name
+    m = c_funcptr_name_re.match(name)
+    if m:
+        name = m.group(1)
     if not arglist:
         if desctype == 'cfunction':
             # for functions, add an empty parameter list
@@ -352,7 +357,7 @@ def desc_directive(desctype, arguments, options, content, lineno,
                 signode['ids'].append(fullname)
                 signode['first'] = (not names)
                 state.document.note_explicit_target(signode)
-                env.note_descref(fullname, desctype)
+                env.note_descref(fullname, desctype, lineno)
             names.append(name)
 
             env.note_index_entry('single',
@@ -664,10 +669,10 @@ def literalinclude_directive(name, arguments, options, content, lineno,
     if not state.document.settings.file_insertion_enabled:
         return [state.document.reporter.warning('File insertion disabled', line=lineno)]
     env = state.document.settings.env
-    fn = arguments[0]
+    rel_fn = arguments[0]
     source_dir = path.dirname(path.abspath(state_machine.input_lines.source(
         lineno - state_machine.input_offset - 1)))
-    fn = path.normpath(path.join(source_dir, fn))
+    fn = path.normpath(path.join(source_dir, rel_fn))
 
     try:
         f = open(fn)
@@ -683,6 +688,7 @@ def literalinclude_directive(name, arguments, options, content, lineno,
             retnode['language'] = options['language']
         if 'linenos' in options:
             retnode['linenos'] = True
+        state.document.settings.env.note_dependency(rel_fn)
     return [retnode]
 
 literalinclude_directive.options = {'linenos': directives.flag,
